﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.IO;

namespace wanshu_api_demo
{
	
	public static class HttpUtility
	{
		const string DEFAULT_USER_AGENT = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)";
		
		private static readonly Encoding encoding = Encoding.UTF8;
		
		//请求连接限时
		const int CON_TIMEOUT = 8 * 1000;
	
		//数据传输限时
		const int READ_WRITE_TIMEOUT = 8 * 1000;
	
		static HttpUtility()
		{
			ServicePointManager.DefaultConnectionLimit = 512;
			ServicePointManager.Expect100Continue = false;
		}
  
  
		/// <summary>    
		/// 创建POST方式的HTTP请求  
		/// </summary>    
		/// <param name="url">请求的URL</param>    
		/// <param name="parameters">随同请求POST的参数名称及参数值字典</param>    
		/// <returns>返回的字符串</returns>  
		public static string Post(string url, IDictionary<string,string> parameters)
		{  
			if (string.IsNullOrEmpty(url)) {  
				throw new ArgumentNullException("url");  
			}  
		
			if (null == parameters) {  
				throw new ArgumentNullException("parameters");  
			}  

			
			HttpWebResponse response = MultipartFormDataPost(url,parameters);

			return OpenReadWithHttps(response);
		}
	
		private static HttpWebRequest getRequest(string url, string method)
		{
			HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
			request.Method = method;//传输方式  
			//request.ContentType = "application/x-www-form-urlencoded";//协议                  
			request.UserAgent = DEFAULT_USER_AGENT;//请求的客户端浏览器信息,默认IE                  
			request.Timeout = CON_TIMEOUT;//连接超时时间	
			request.ReadWriteTimeout = READ_WRITE_TIMEOUT;//读写超时时间
		
			//如果是发送HTTPS请求    
			if (url.StartsWith("https", StringComparison.OrdinalIgnoreCase)) {  
				ServicePointManager.ServerCertificateValidationCallback += CheckValidationResult;  
				request.ProtocolVersion = HttpVersion.Version10;  
			} 
			return request;
		}
	
  
		//验证服务器证书回调自动验证
		private static bool CheckValidationResult(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors)
		{  
			return true; //总是接受    
		}
  
		/// <summary>  
		/// 获取数据  
		/// </summary>  
		/// <param name="httpWebResponse">响应对象</param>  
		/// <returns>返回字符串</returns>  
		private static string OpenReadWithHttps(HttpWebResponse httpWebResponse)
		{  
			if (httpWebResponse == null) {
				return null;
			}
		
			Stream responseStream = null;  
			StreamReader sReader = null;  
			string value = null;  
			
			try { 	
				if (HttpStatusCode.OK.Equals(httpWebResponse.StatusCode)) {
					// 获取响应流  
					responseStream = httpWebResponse.GetResponseStream();  
					// 对接响应流(以"utf-8"字符集)  
					sReader = new StreamReader(responseStream, encoding);  
					// 开始读取数据  
					value = sReader.ReadToEnd();  
				}
			} catch (Exception ex) {  
				Console.WriteLine(ex); //正式项目中请改为log打印
			} finally {  
				//强制关闭  
				if (sReader != null) {  
					sReader.Close();  
				}  
				if (responseStream != null) {  
					responseStream.Close();  
				}  
				if (httpWebResponse != null) {  
					httpWebResponse.Close();  
				}  
			}  
  
			return value;  
		}
		

		
	    public static HttpWebResponse MultipartFormDataPost(string postUrl, IDictionary<string, string> postParameters)
	    {
	        string formDataBoundary = String.Format("----------{0:N}", Guid.NewGuid());
	        string contentType = "multipart/form-data; boundary=" + formDataBoundary;
	 
	        byte[] formData = GetMultipartFormData(postParameters, formDataBoundary);
	 
	        return PostForm(postUrl, contentType, formData);
	    }
	    
	    
	    private static HttpWebResponse PostForm(string postUrl, string contentType, byte[] formData)
	    {
	    	HttpWebRequest request = getRequest(postUrl,"POST");
	 
	        if (request == null)
	        {
	            throw new NullReferenceException("request is not a http request");
	        }
	 
	        // Set up the request properties.
	        request.Method = "POST";
	        request.ContentType = contentType;
	        //request.UserAgent = userAgent;
	        request.CookieContainer = new CookieContainer();
	        request.ContentLength = formData.Length;
	 
	        // You could add authentication here as well if needed:
	        // request.PreAuthenticate = true;
	        // request.AuthenticationLevel = System.Net.Security.AuthenticationLevel.MutualAuthRequested;
	        // request.Headers.Add("Authorization", "Basic " + Convert.ToBase64String(System.Text.Encoding.Default.GetBytes("username" + ":" + "password")));
	 
	        // Send the form data to the request.
	        using (Stream requestStream = request.GetRequestStream())
	        {
	            requestStream.Write(formData, 0, formData.Length);
	            requestStream.Close();
	        }
	 
	        return request.GetResponse() as HttpWebResponse;
	    }
	 
	    private static byte[] GetMultipartFormData(IDictionary<string, string> postParameters, string boundary)
	    {
	        Stream formDataStream = new System.IO.MemoryStream();
	        bool needsCLRF = false;
	 
	        foreach (var param in postParameters)
	        {
	            // Thanks to feedback from commenters, add a CRLF to allow multiple parameters to be added.
	            // Skip it on the first parameter, add it to subsequent parameters.
	            if (needsCLRF)
	                formDataStream.Write(encoding.GetBytes("\r\n"), 0, encoding.GetByteCount("\r\n"));
	 
	            needsCLRF = true;
                string postData = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}",
                    boundary,
                    param.Key,
                    param.Value);
                formDataStream.Write(encoding.GetBytes(postData), 0, encoding.GetByteCount(postData));
	        }
	 
	        // Add the end of the request.  Start with a newline
	        string footer = "\r\n--" + boundary + "--\r\n";
	        formDataStream.Write(encoding.GetBytes(footer), 0, encoding.GetByteCount(footer));
	 
	        // Dump the Stream into a byte[]
	        formDataStream.Position = 0;
	        byte[] formData = new byte[formDataStream.Length];
	        formDataStream.Read(formData, 0, formData.Length);
	        formDataStream.Close();
	 
	        return formData;
	    }
 
	  
		
	}
	
}