package main

import (
	"bytes"
	"encoding/json"
	"fmt"
	"log"
	"mime/multipart"
	"net/http"
	"crypto/cipher"
	"encoding/base64"
	"crypto/des"
)


func Decrypt(data []byte, key []byte) (string, error) {
	block, err := des.NewCipher(key)
	if err != nil {
		return "", err
	}
	blockMode := cipher.NewCBCDecrypter(block, []byte{1,2,3,4,5,6,7,8})
	out := make([]byte, len(data))
	blockMode.CryptBlocks(out, data)
	out = PKCS5UnPadding(out)
	return string(out), nil
}

func PKCS5UnPadding(origData []byte) []byte {
	length := len(origData)
	unpadding := int(origData[length-1])
	return origData[:(length - unpadding)]
}

func Base64Decode(src []byte) ([]byte, error) {
	return base64.StdEncoding.DecodeString(string(src))
}


type Params struct {
	APPid string
	AccessToken string
	Telecom string
	Timestamp string
	Randoms string
	Version string
	Device string
	Sign string
}

type Response struct {
	Code    string                    `json:"code"`
	Message string                 `json:"message"`
	Data    Data `json:"data"`
}

type Data struct {
	TradeNo string `json:"tradeNo"`
	MobileName string `json:"mobileName"`
	ChargeStatus string `json:"chargeStatus"`
	Fanqizha string `json:"fanqizha"`
	tag string `json:"tag"`
}

var urlmap = map[string]string{
	"CMCC":"https://api.253.com/open/flashsdk/mobile-query-m",
	"CUCC":"https://api.253.com/open/flashsdk/mobile-query-u",
	"CTCC":"https://api.253.com/open/flashsdk/mobile-query-t",
}


func postForm(data string)(body string) {
	var jsonBytes []byte
	jsonBytes = []byte(data)
	p := new(Params)
	json.Unmarshal(jsonBytes,p)
	req,err := newMultipartRequest(urlmap[p.Telecom],*p)
	if err != nil {
		log.Fatal(err)
	}
	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		log.Fatal(err)
	} else {
		body := &bytes.Buffer{}
		_, err := body.ReadFrom(resp.Body)
		if err != nil {
			log.Fatal(err)
		}
		resp.Body.Close()
		if resp.StatusCode == 200 {
			return body.String();
		}
	}
	return "";
}

func newMultipartRequest(url string, p Params) (*http.Request, error) {
	body := &bytes.Buffer{}
	writer := multipart.NewWriter(body)
	writer.WriteField("appId",p.APPid)
	writer.WriteField("accessToken",p.AccessToken)
	writer.WriteField("telecom",p.Telecom)
	writer.WriteField("timestamp",p.Timestamp)
	writer.WriteField("randoms",p.Randoms)
	writer.WriteField("version",p.Version)
	writer.WriteField("device",p.Device)
	writer.WriteField("sign",p.Sign)
	writer.Close()
	req, error := http.NewRequest("POST", url, body)
	req.Header.Set("Content-Type", writer.FormDataContentType())
	return req, error
}

func main(){
	//data为SDK实际返回的参数
	data :=`{"appId":"xxxxxxxx","sign":"EG4coN0v0P8CGTXxd1WucDMGes0=","timestamp":"1560309724312","randoms":"e374c9ae-7338-473c-a4a0-c3032ec3936a","device":"device=HUAWEI","accessToken":"d9e430aa2c8d449594dd064ef5053249","telecom":"CTCC","version":"2.1.3"}`;
	//应用对应的APPKEY
	appKey :="xxxxxxxx";
	//发送HTTP POST请求
	body := postForm(data)
	if body == "" {
		fmt.Println("接口调用失败")
		return
	}
	//解析返回结果
	res := Response{}
	err := json.Unmarshal([]byte(body), &res)
	if err != nil {
		fmt.Println(err)
	}
	if res.Code == "200000" {
		data := res.Data
		mobileName :=data.MobileName
		tradeNo :=data.TradeNo
		//解密手机号
		fmt.Println("手机号密文:"+mobileName+",交易流水号:"+tradeNo)
		if mobileName!="" {
			enbyte, _ := Base64Decode([]byte(mobileName))
			mobile, _ := Decrypt(enbyte, []byte(appKey))
			fmt.Println("手机号："+mobile)
		}
	}else{
		fmt.Println("接口调用失败，返回的报错信息：",body)
	}
}



